@@ -2,6 +2,7 @@ import json
 import requests
 from typing import Type, TypeVar, Dict, Callable, List, Union, Any, Optional, Sequence
 import io
+import functools
 
 from dataclasses_jsonschema import ValidationError, JsonSchemaMixin
 from PIL import Image, UnidentifiedImageError  # type: ignore
@@ -20,6 +21,26 @@ T = TypeVar('T', bound=JsonSchemaMixin)
 S = TypeVar('S', str, int, float, bool)
 
 OptionalData = Optional[Union[JsonSchemaMixin, Sequence[JsonSchemaMixin]]]
+ParamsDict = Optional[Dict[str, Any]]
+
+SESSION = requests.session()
+
+
+def handle_exceptions(exception_type: Type[Arcor2Exception] = Arcor2Exception, message: Optional[str] = None):
+    def _handle_exceptions(func):
+        @functools.wraps(func)
+        def wrapper(*args, **kwargs):
+
+            try:
+                return func(*args, **kwargs)
+            except RestException as e:
+                if message is not None:
+                    raise exception_type(message, str(e)) from e
+                else:
+                    raise exception_type(e.message) from e
+
+        return wrapper
+    return _handle_exceptions
 
 
 def convert_keys(d: Union[Dict, List], func: Callable[[str], str]) -> Union[Dict, List]:
@@ -46,13 +67,16 @@ def handle_response(resp: requests.Response) -> None:
         try:
             resp_body = json.loads(resp.content)
         except json.JSONDecodeError:
-            resp_body = resp.content
+            raise RestException(resp.content.decode("utf-8"), str(e)) from e
 
-        raise RestException(f"Status code: {resp.status_code}, "f"body: {resp_body}.", str(e))
+        try:
+            raise RestException(resp_body["message"], str(e)) from e
+        except KeyError:
+            raise RestException(str(resp_body), str(e)) from e
 
 
 def _send(url: str, op: Callable, data: OptionalData = None,
-          params: Optional[Dict] = None, get_response=False) -> Union[None, Dict, List]:
+          params: ParamsDict = None, get_response=False) -> Union[None, Dict, List]:
 
     if data:
         if isinstance(data, list):
@@ -73,7 +97,7 @@ def _send(url: str, op: Callable, data: OptionalData = None,
         resp = op(url, data=json.dumps(d), timeout=TIMEOUT, headers={'Content-Type': 'application/json'},
                   params=params)
     except requests.exceptions.RequestException as e:
-        raise RestException(f"Catastrophic system error.", str(e))
+        raise RestException(f"Catastrophic system error.", str(e)) from e
 
     handle_response(resp)
 
@@ -83,24 +107,24 @@ def _send(url: str, op: Callable, data: OptionalData = None,
     try:
         return convert_keys(json.loads(resp.text), camel_case_to_snake_case)
     except (json.JSONDecodeError, TypeError) as e:
-        raise RestException("Invalid JSON.", str(e))
+        raise RestException("Invalid JSON.", str(e)) from e
 
 
-def post(url: str, data: JsonSchemaMixin, params: Optional[Dict] = None):
-    _send(url, requests.post, data, params)
+def post(url: str, data: JsonSchemaMixin, params: ParamsDict = None):
+    _send(url, SESSION.post, data, params)
 
 
 def put_returning_primitive(url: str, desired_type: Type[S], data: OptionalData = None,
-                            params: Optional[Dict] = None) -> S:
+                            params: ParamsDict = None) -> S:
 
     try:
-        return desired_type(_send(url, requests.put, data, params, get_response=True))  # type: ignore
+        return desired_type(_send(url, SESSION.put, data, params, get_response=True))  # type: ignore
     except ValueError as e:
-        raise RestException(e)
+        raise RestException(e) from e
 
 
-def put(url: str, data: OptionalData = None, params: Optional[Dict] = None, data_cls: Type[T] = None) -> T:
-    ret = _send(url, requests.put, data, params, get_response=data_cls is not None)  # type: ignore
+def put(url: str, data: OptionalData = None, params: ParamsDict = None, data_cls: Type[T] = None) -> T:
+    ret = _send(url, SESSION.put, data, params, get_response=data_cls is not None)  # type: ignore
 
     if not data_cls:
         return None  # type: ignore
@@ -109,13 +133,13 @@ def put(url: str, data: OptionalData = None, params: Optional[Dict] = None, data
         return data_cls.from_dict(ret)  # type: ignore
     except ValidationError as e:
         print(f'{data_cls.__name__}: validation error "{e}" while parsing "{data}".')
-        raise RestException("Invalid data.", str(e))
+        raise RestException("Invalid data.", str(e)) from e
 
 
 def put_returning_list(url: str, data: OptionalData = None,
-                       params: Optional[Dict] = None, data_cls: Type[T] = None) -> List[T]:
+                       params: ParamsDict = None, data_cls: Type[T] = None) -> List[T]:
 
-    ret = _send(url, requests.put, data, params, get_response=data_cls is not None)  # type: ignore
+    ret = _send(url, SESSION.put, data, params, get_response=data_cls is not None)  # type: ignore
 
     if not data_cls:
         return []  # type: ignore
@@ -128,7 +152,7 @@ def put_returning_list(url: str, data: OptionalData = None,
             d.append(data_cls.from_dict(dd))
         except ValidationError as e:
             print(f'{data_cls.__name__}: validation error "{e}" while parsing "{ret}".')
-            raise RestException("Invalid data.", str(e))
+            raise RestException("Invalid data.", str(e)) from e
 
     return d
 
@@ -136,14 +160,14 @@ def put_returning_list(url: str, data: OptionalData = None,
 def delete(url: str):
 
     try:
-        resp = requests.delete(url, timeout=TIMEOUT)
+        resp = SESSION.delete(url, timeout=TIMEOUT)
     except requests.exceptions.RequestException as e:
-        raise RestException(f"Catastrophic system error.", str(e))
+        raise RestException(f"Catastrophic system error.", str(e)) from e
 
     handle_response(resp)
 
 
-def get_data(url: str, body: Optional[JsonSchemaMixin] = None, params: Optional[Dict] = None) -> Union[Dict, List]:
+def get_data(url: str, body: Optional[JsonSchemaMixin] = None, params: ParamsDict = None) -> Union[Dict, List]:
 
     data = _get(url, body, params)
 
@@ -153,7 +177,7 @@ def get_data(url: str, body: Optional[JsonSchemaMixin] = None, params: Optional[
     return convert_keys(data, camel_case_to_snake_case)
 
 
-def _get_response(url: str, body: Optional[JsonSchemaMixin] = None, params: Optional[Dict] = None) -> requests.Response:
+def _get_response(url: str, body: Optional[JsonSchemaMixin] = None, params: ParamsDict = None) -> requests.Response:
 
     if body is None:
         body_dict = {}  # type: ignore
@@ -166,24 +190,23 @@ def _get_response(url: str, body: Optional[JsonSchemaMixin] = None, params: Opti
         params = {}
 
     try:
-        resp = requests.get(url, timeout=TIMEOUT, data=body_dict, params=params)
+        resp = SESSION.get(url, timeout=TIMEOUT, data=body_dict, params=params)
     except requests.exceptions.RequestException as e:
-        print(e)
-        raise RestException(f"Catastrophic system error.", str(e))
+        raise RestException(f"Catastrophic system error.", str(e)) from e
 
     handle_response(resp)
 
     return resp
 
 
-def _get(url: str, body: Optional[JsonSchemaMixin] = None, params: Optional[Dict] = None) -> Any:
+def _get(url: str, body: Optional[JsonSchemaMixin] = None, params: ParamsDict = None) -> Any:
 
     resp = _get_response(url, body, params)
 
     try:
         return json.loads(resp.text)
     except (json.JSONDecodeError, TypeError) as e:
-        raise RestException("Invalid JSON.", str(e))
+        raise RestException("Invalid JSON.", str(e)) from e
 
 
 def get_image(url: str) -> Image.Image:
@@ -192,22 +215,23 @@ def get_image(url: str) -> Image.Image:
     try:
         return Image.open(io.BytesIO(_get_response(url).content))
     except (UnidentifiedImageError, TypeError) as e:
-        raise RestException(e)
+        raise RestException("Invalid image.", str(e)) from e
 
 
 def get_primitive(url: str, desired_type: Type[S], body: Optional[JsonSchemaMixin] = None,
-                  params: Optional[Dict] = None) -> S:
+                  params: ParamsDict = None) -> S:
 
     value = _get(url, body, params)
 
     try:
         return desired_type(value)
     except ValueError as e:
-        raise RestException(e)
+        raise RestException("Invalid primitive.", str(e)) from e
 
 
-def get_list(url: str, data_cls: Type[T]) -> List[T]:
-    data = get_data(url)
+def get_list(url: str, data_cls: Type[T], body: Optional[JsonSchemaMixin] = None, params: ParamsDict = None) -> List[T]:
+
+    data = get_data(url, body, params)
 
     ret: List[T] = []
 
@@ -216,7 +240,7 @@ def get_list(url: str, data_cls: Type[T]) -> List[T]:
             ret.append(data_cls.from_dict(val))
         except ValidationError as e:
             print(f'{data_cls.__name__}: validation error "{e}" while parsing "{data}".')
-            raise RestException("Invalid data.", str(e))
+            raise RestException("Invalid data.", str(e)) from e
 
     return ret
 
@@ -233,25 +257,25 @@ def get_list_primitive(url: str, desired_type: Type[S]) -> List[S]:
     return ret
 
 
-def get(url: str, data_cls: Type[T], body: Optional[JsonSchemaMixin] = None) -> T:
+def get(url: str, data_cls: Type[T], body: Optional[JsonSchemaMixin] = None, params: ParamsDict = None) -> T:
 
-    data = get_data(url, body)
+    data = get_data(url, body, params)
 
     assert isinstance(data, dict)
 
     try:
         return data_cls.from_dict(data)
     except ValidationError as e:
-        raise RestException("Invalid data.", str(e))
+        raise RestException("Invalid data.", str(e)) from e
 
 
 def download(url: str, path: str) -> None:
     # TODO check content type
 
     try:
-        r = requests.get(url, allow_redirects=True)
+        r = SESSION.get(url, allow_redirects=True)
     except requests.exceptions.RequestException as e:
-        raise RestException("Download of file failed.", str(e))
+        raise RestException("Download of file failed.", str(e)) from e
 
     handle_response(r)
 
